%global momorel 3

## This package understands the following switches:
## --with[out] ssp  ...  enable/disable SSP; default depends
##                       on target architecture
## --with dynamic   ...  enable dynamic lib support

%global runtests        0
%global snapshot	20090228
%global pkglibdir	%{_prefix}/lib/dietlibc

%ifarch %{ix86} x86_64
%bcond_without		ssp
%else
%bcond_with		ssp
%endif

%ifarch %{ix86} x86_64 arm
%bcond_with		dynamic
%endif

%ifarch %{ix86}
%global target_cpu	i386
%else
%global target_cpu	%{_target_cpu}
%endif

Summary:	Small libc implementation
Name:		dietlibc
Version:	0.31
Release:	%{snapshot}.%{momorel}m%{?dist}
License:	GPLv2
Group:		Development/Libraries
URL:		http://www.fefe.de/dietlibc/
%if !0%{?snapshot:1}
Source0:	http://www.kernel.org/pub/linux/libs/%{name}/%{name}-%{version}.tar.bz2
Source1:	http://www.fefe.de/%{name}/%{name}-%{version}.tar.bz2.sig
%else
# generated by 'make cvs-sources [CVS_DATE=....]'
Source0:	%{name}-%{version}.%{snapshot}.tar.bz2
%endif
Source10:	runtests-X.sh
Patch1:		dietlibc-0.28-setpriority.patch
Patch10:	dietlibc-0.29-scall.patch
Patch20:	dietlibc-0.29-nostackprotector.patch
Patch30:	dietlibc-0.30-longdouble.patch
Patch31:	dietlibc-0.31-defpath.patch
Patch32:	dietlibc-0.31-stacksmash.patch
Patch33:	dietlibc-0.31-stacksmash-dyn.patch
Patch41:	dietlibc-0.31.20080212-teststdout.patch
Patch43:	dietlibc-0.31-pagesize.patch
Patch44:	dietlibc-0.31-printFG.patch
Patch46:	dietlibc-0.31-testsuite.patch
Patch47:	dietlibc-0.31-lcctime.patch
Patch48:	dietlibc-0.31-implicitfunc.patch
Patch49:	dietlibc-0.31-noreturn.patch
Patch50:	dietlibc-0.31-no-stack-protector.patch
BuildRoot:      %{_tmppath}/%{name}-%{version}-%{release}-root-%(%{__id_u} -n)
%{?with_dynamic:Requires:	dietlibc-lib = %version-%release}
%{!?with_dynamic:Obsoletes:	dietlibc-lib < %version-%release}

Requires:	%{name}-devel = %{version}-%{release}
BuildRequires:	gdb

%package devel
Summary:	dietlibc development files
Group:		Development/Libraries
Requires:	%{name} = %{version}-%{release}
Requires:	%{name}-header = %{version}-%{release}

%package header
Summary:	dietlibc header files
Group:		Development/Libraries
Requires:	%{name} = %{version}-%{release}
Requires(pre):	%{name}-devel = %{version}-%{release}
Requires:	%{name}-devel = %{version}-%{release}
#BuildArch:	noarch

%package lib
Summary:	Dynamic libraries for dietlibc
Group:		System Environment/Libraries
Conflicts:	%{name} < %{version}-%{release}
Conflicts:	%{name} > %{version}-%{release}

%description
The diet libc is a libc that is optimized for small size. It can be
used to create small statically linked binaries for Linux on alpha,
arm, hppa, ia64, i386, mips, s390, sparc, sparc64, ppc and x86_64.

%description devel
The diet libc is a libc that is optimized for small size. It can be
used to create small statically linked binaries for Linux on alpha,
arm, hppa, ia64, i386, mips, s390, sparc, sparc64, ppc and x86_64.

This package contains the object files for dietlibc.

%description header
The diet libc is a libc that is optimized for small size. It can be
used to create small statically linked binaries for Linux on alpha,
arm, hppa, ia64, i386, mips, s390, sparc, sparc64, ppc and x86_64.

This package contains the header files for dietlibc.

%description lib
The diet libc is a libc that is optimized for small size. It can be
used to create small statically linked binaries for Linux on alpha,
arm, hppa, ia64, i386, mips, s390, sparc, sparc64, ppc and x86_64.

This package contains the dynamic libraries for dietlibc.


%prep
%setup -q %{?snapshot:-n %name-%version.%snapshot}
%patch1  -p1 -b .nice
%patch10 -p1 -b .scall
%patch30 -p1 -b .longdouble
%patch31 -p1 -b .defpath
%patch32 -p1 -b .stacksmash
%patch33 -p1 -b .stacksmash-dyn

%patch41 -p1 -b .teststdout
%patch43 -p1 -b .pagesize
%patch44 -p1 -b .printFG
%patch46 -p1 -b .testsuite
%patch47 -p1 -b .lcctime
%patch48 -p1 -b .implicitfunc
%patch49 -p1 -b .noreturn
%patch50 -p1 -b .no-stack-protector

%if %{without ssp}
%patch20 -p1 -b .nostackprotector
sed -i -e 's!^#define WANT_SSP$!// \0!g;
	   s!.*\(#define WANT_STACKGAP\).*!\1!g' dietfeatures.h
%global xtra_fixcflags	-fno-stack-protector
%else
%global xtra_fixcflags	%nil
%endif

%ifarch %{ix86}
sed -i \
	-e '/#define WANT_DYN_PAGESIZE/{c\'	\
	-e '#define WANT_ELFINFO' 		\
	-e '}'					\
	dietfeatures.h
%endif

sed -i -e 's!strip !: !g' Makefile

%global fixcflags	-fomit-frame-pointer -fno-exceptions -fno-asynchronous-unwind-tables %{xtra_fixcflags} -Os -g3 -Werror-implicit-function-declaration
%global basemakeflags	prefix=%{pkglibdir} BINDIR=%{_bindir} MAN1DIR=%{_mandir}/man1 COMMENT=: CFLAGS="$RPM_OPT_FLAGS %{fixcflags} $XTRA_CFLAGS" PDIET=%{pkglibdir}
%global makeflags	%{basemakeflags}

for i in `find test -name 'runtests.sh'`; do
	ln -s %{SOURCE10} `dirname $i`/runtests-X.sh
done


%build
make %{makeflags} all %{?_smp_mflags}

# 'dyn' target is not SMP safe
%{?with_dynamic:make %makeflags dyn}


%install
rm -rf %{buildroot}

install -d -m755 %{buildroot}/etc
make %{makeflags} DESTDIR=%{buildroot} install

ln -s lib-%{_arch} %{buildroot}%{pkglibdir}/lib-%{_arch}-%{_vendor}

chmod a-x %{buildroot}%{pkglibdir}/lib-*/*.o
rm -f %{buildroot}%{_bindir}/dnsd

%if %{runtests}
%check
XTRA_CFLAGS='-fno-builtin'
make %{makeflags} -C test      all %{?_smp_mflags} DIET=$(echo `pwd`/bin-*/diet) -k || :
make %{makeflags} -C test/inet all %{?_smp_mflags} DIET=$(echo `pwd`/bin-*/diet)    || :

cd test
ulimit -m $[ 128*1024 ] -v $[ 256*1024 ] -d $[ 128*1024 ]

bash ./runtests-X.sh
%endif # %%{runtests}


%clean
rm -rf %{buildroot}

%files
%defattr(-,root,root,-)
%doc AUTHOR BUGS CAVEAT CHANGES COPYING FAQ PORTING README*
%doc SECURITY THANKS TODO
%doc %{_mandir}/*/*
%{_bindir}/*


%files header
%defattr(-,root,root,-)
%pkglibdir/include


%files devel
%defattr(-,root,root,-)
%{pkglibdir}
%exclude %{pkglibdir}/include
%{?with_dynamic:%exclude %{pkglibdir}/*/*.so}


%if %{with dynamic}
%files lib
%defattr(-,root,root,-)
%config(noreplace) %{_sysconfdir}/*
%dir %{pkglibdir}
%dir %{pkglibdir}/lib-*
%{pkglibdir}/lib-%{target_cpu}/*.so
%endif


%changelog
* Tue Aug 31 2010 Yohsuke Ooi <meke@momonga-linux.org>
- (0.31-20090228.3m)
- full rebuild for mo7 release

* Sat Nov 14 2009 NAKAYA Toshiharu <nakaya@momonga-linux.org>
- (0.31-20090228.2m)
- rebuild against gcc-4.4 and glibc-2.11

* Tue Mar 17 2009 NAKAYA Toshiharu <nakaya@momonga-linux.org>
- (0.31-20090228.1m)
- sync with Rawhide (0.31-8.20090228)

* Wed Jan 21 2009 NAKAYA Toshiharu <nakaya@momonga-linux.org>
- (0.31-20080517.3m)
- rebuild against rpm-4.6

* Fri Jul 25 2008 Ichiro Nakai <ichiro@n.email.ne.jp>
- (0.31-5.20080517.2m)
- fix up spec file to enable build

* Fri Jul 25 2008 TABUCHI Takaaki <tab@momonga-linux.org>
- (0.31-5.20080517.1m)
- sync Fedora
- warning: no exec make test for error

* Sun May 18 2008 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.31-5.20080517
- updated to 20080517 snapshot
- use patches from git repository

* Sun May 18 2008 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.31-5.20080409
- fixed __signalfd() prototype

* Sat Apr 19 2008 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.31-4.20080409
- update -pagesize patch

* Mon Apr 14 2008 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.31-3.20080409
- removed debug stuff from specfile
- updated patches to work with new isinf() behavior of gcc 4.3

* Sun Apr 13 2008 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.31-2.20080409
- added patch for dynamic PAGE_SIZE support
- fixed/enhanced testsuite and removed the '|| :' in %%check
- improved/fixed floating point support *printf(3)

* Thu Apr 10 2008 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.31-1.20080409
- updated to CVS snapshot 20080409

* Thu Apr 03 2008 TABUCHI Takaaki <tab@momonga-linux.org>
- (0.27-7m)
- rebuild against gcc43

* Fri Feb 22 2008 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.31-1.20080221
- updated to CVS snapshot 20080221; removed most of the last patches
  as they are now in upstream
- moved files into platform neutral /usr/lib dir (not using %%_lib or
  %%_libdir macro)
- added -devel subpackage due to multiarch issues; main package contains
  only the 'diet' binary plus some tools while -devel holds all the
  header and object files.
- fixed optimized memcpy(3)

* Wed Feb 13 2008 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.31-1.20080212
- updated to CVS snapshot 20080212
- fixed printf regression for '%+04i' style formats
- added %%check and run a testsuite; it does not succeed now so it is
  for informational purposes only...
- added bunch of patches to fixes big-endian issues in string routines

* Sun Sep 23 2007 Hiromasa YOSHIMOTO <y@momonga-linux.org>
- (0.27-6m)
- revised spec for debuginfo

* Sat Sep  1 2007 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.31-1
- updated to 0.31
- removed the no-stack-protector bits for i386 and x86_64 archs
- improved stack-smash code a little bit
- disabled dynamic lib for all arches
- made objects non-executable to avoid "No build ID note" errors

* Wed Jan 17 2007 David Woodhouse <dwmw2@infradead.org> 0.30-4
- Bump release to be higher than unexplained 0.30-3.fc6

* Wed Jan 17 2007 David Woodhouse <dwmw2@infradead.org> 0.30-3
- Apply workaround for GCC PR26374 to build on PPC again (#182118)

* Fri Sep 15 2006 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.30-2
- rebuilt

* Sun Jul  9 2006 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.30-1
- updated to 0.30
- removed cross-arch support
- disable (non-working) SSP support; enable old stackgap code instead of

* Sat Feb 18 2006 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.29-6
- added '-Os' to the CFLAGS
- exclude PPC arch due to strange compilation errors

* Sun Oct 30 2005 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.29-4
- added '-fno-stack-protector' to the CFLAGS

* Sun Oct 30 2005 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.29-3
- removed '-fstack-protector' from CLI because it uses __stack_chk_fail
  which is not provided by dietlibc

* Thu Oct 27 2005 Hiroyuki Koga <kuma@momonga-linux.org>
- (0.27-5m)
- add gcc4 patch (Patch103)

* Thu May 26 2005 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.29-2
- rebuilt

* Tue May 24 2005 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.29-1
- updated to 0.29
- unset some compiler options which cause errors with gcc4
- rediffed -scall patch
- removed most patches as already in upstream

* Thu May 19 2005 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.28-6
- use %%dist instead of %%disttag
- package some more %%doc files (reported by Chris Ricker)
- add %%release to the BuildRoot (reported by Chris Ricker)
- fixed URLs of the sources (reported by Chris Ricker)

* Sun May  1 2005 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.28-5
- added Herbert Poetzl's syscall(2) implementation for x86_64 and
  sparc* architectures (see
  http://vserver.13thfloor.at/Experimental/delta-diet.diff); rediffed
  the .call patch
- catch the case when syscall(2) is used on archs where it is not
  implemented yet
- define MADV_* constants on HPPA and IA64 archs
- fix *truncate64() handling on 64bit archs

* Fri Mar  4 2005 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 0.28-4
- rebuild
- fixed compilation with gcc4
- made nice(2) SUSv3 compliantly on x86_64 and other platforms (patch
  provided by Colin Hill)
- build the dynamic libs for x86, x86_64 and arm only
- fixed PPC issues in ISO mode (do not use the 'powerpc' macro but the
  '__powerpc__' one)
- added some O_* -flags for the HPPA arch
- use more modern flags for '-Os'

* Fri Feb 11 2005 Dai OKUYAMA <dai@ouchi.nahi.to>
- (0.27-4m)
- enable x86_64.
  - FIXME: creating shared object failed. orz

* Tue Feb  1 2005 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 1:0.28-1
- updated to 0.28
- unset LD_RUN_PATH which would result in empty rpaths else
- removed the waitpid patch; it is solved upstream

* Wed Jan 26 2005 Tsutomu Yasuda <tom@tom.homelinux.org>
- (0.27-3m)
  applied progname patch
  
* Tue Jan 25 2005 Toru Hoshina <t@momonga-linux.org>
- (0.27-2m)
- patches from FC3.

* Tue Aug 24 2004 Tsutomu Yasuda <tom@tom.homelinux.org>
- (0.27-1m)
  update to 0.27

* Tue Aug 17 2004 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 1:0.27-1
- updated to 0.27

* Fri Jun 11 2004 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 1:0.26-1
- updated to 0.26

* Tue Jun  1 2004 Masahiro Takahata <takahata@momonga-linux.org>
- (0.22-3m)
- change URI

* Fri Apr  9 2004 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 1:0.25-1
- updated to 0.25

* Tue Mar 15 2004 Toru Hoshina <t@momonga-linux.org>
- (0.22-2m)
- revised spec for enabling rpm 4.2.
- modified string.h for momos gcc.

* Sat Jan 17 2004 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 1:%VERSION.90.%DATE}-5
- renewed -snprintf patch

* Tue Jan  6 2004 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 1:%VERSION.90.%DATE}-4
- applied -snprintf patch (from fedora dietlibc-0.24-2 package)

* Tue Dec 30 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 1:%VERSION.90.%DATE}-2
- added C99 patch

* Thu Dec  4 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 1:0.24-1
- updated to 0.24

* Tue Sep 16 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> - 1:0.23-1
- updated to 0.23

* Thu Feb 20 2003 Tsutomu Yasuda <tom@tom.homelinux.org>
- (0.22-1m)
  update to 0.22

* Sat Nov 16 2002 KOMATSU Shinichiro <koma2@jiro.c.u-tokyo.ac.jp>
- (0.21-4m)
- support __progname on alpha

* Sat Nov 16 2002 KOMATSU Shinichiro <koma2@jiro.c.u-tokyo.ac.jp>
- (0.21-3m)
- add dietlibc-0.21-libgccopt.patch
  (when linking, add -lgcc after libc.a) 
- add dietlibc-0.21-progname.patch
  (set __progname. currently x86 only)

* Fri Nov  8 2002 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> 1:0.21-1
- updated to 0.21
- removed unpackaged files in %%install-stage

* Mon Oct 14 2002 KOMATSU Shinichiro <koma2@jiro.c.u-tokyo.ac.jp>
- (0.21-2m)
- successfully build on alpha (ExclusiveArch really needed?)

* Tue Oct  8 2002 Kazuhiko <kazuhiko@fdiary.net>
- (0.21-1m)
- minor feature enhancements
- build on ppc

* Sun Aug 18 2002 Motonobu Ichimura <famao@momonga-linux.org>
- (0.20-1m)
- up to 0.20

* Sun Aug 11 2002 Kazuhiko <kazuhiko@fdiary.net>
- (0.19-1m)
- security fixes

* Wed Jul 17 2002 Kazuhiko <kazuhiko@fdiary.net>
- (0.18-1m)
- add some documents

* Thu Jul 11 2002 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de> 1:0.18-1
- updated to 0.18

* Thu Jul  4 2002 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
- Initial build.

* Fri Apr  5 2002 Toru Hoshina <t@kondara.org>
- (0.16-4k)
- patch ga naiyo-n (T_T)

* Thu Apr  4 2002 Tsutomu Yasuda <tom@kondara.org>
- (0.16-2k)
  update to 0.16

* Sun Mar 31 2002 Toru Hoshina <t@kondara.org>
- (0.15-2k)

* Thu Mar 14 2002 Jeremy Katz <katzj@redhat.com>
- remove the i386 optimized getenv since it seems to break when you have 
  two variables that start out the same (eg LANG and LANGKEY)

* Tue Feb 26 2002 Erik Troan <ewt@redhat.com>
- updated to 0.15 -- no more RH specific patches required

* Wed Feb 20 2002 Jeremy Katz <katzj@redhat.com>
- update to 0.14 which includes a lot of Erik's patches
- include the man page
- munge a few patches
- we want to allow linking against glibc objects

* Mon Feb  4 2002 Bill Nottingham <notting@redhat.com>
- add fdatasync

* Wed Jan 09 2002 Tim Powers <timp@redhat.com>
- automated rebuild

* Sat Dec 15 2001 Erik Troan <ewt@redhat.com>
- fixed putenv to remove environment variables

* Fri Dec 07 2001 Erik Troan <ewt@redhat.com>
- added fnmatch() fix

* Fri Nov 09 2001 Erik Troan <ewt@redhat.com>
- initial packaging
